from sachima import conf
import sachima.sns as sns
import datetime
import functools


def send(func):
    """
    send msg to sns app called dingding
    https://open-doc.dingtalk.com/docs/doc.htm?spm=a219a.7629140.0.0.2vzwCr&treeId=257&articleId=105735&docType=1
    you should conif your token in sachima.yaml
    """
    ERROR_GRP_TOKEN = conf.get("SNS_DINGDING_ERROR_GRP_TOKEN")
    INFO_GRP_TOKEN = conf.get("SNS_DINGDING_INFO_GRP_TOKEN")
    SENDING_STR = conf.get("SNS_DINGDING_SENDING_STR")
    ERRSENT_STR = conf.get("SNS_DINGDING_ERRSENT_STR")

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            # before
            time_str = str(datetime.datetime.now())
            t = SENDING_STR.format(args[0]["handler"], time_str)
            sns.send_dingding(t, t, ERROR_GRP_TOKEN)
            value = func(*args, **kwargs)
            # after
            return value
        except Exception as ex:
            title = ERRSENT_STR.format(
                args[0]["handler"], str(datetime.datetime.now())
            )
            data = str(ex)
            sns.send_dingding(title, title + data, ERROR_GRP_TOKEN)
            sns.send_dingding(title, title + data, INFO_GRP_TOKEN)

    return wrapper



from urllib import request
from urllib import parse
import json
import pandas as pd


def send_request(host, path, webhook, querys, data):
    try:
        url = host + path + "?" + querys
        headers = {"Content-Type": "application/json"}
        req = request.Request(
            url=url, headers=headers, data=json.dumps(data).encode("utf-8")
        )
        response = request.urlopen(req)
        content = response.read().decode("utf-8")
    except:
        raise
    return content


def send_dingding(title, data, webhook):
    host = "https://oapi.dingtalk.com/"
    path = "robot/send"
    querys = "access_token=" + webhook

    jsondata = {
        "msgtype": "markdown",
        "markdown": {"title": title, "text": data},
    }
    return send_request(host, path, webhook, querys, jsondata)



import smtplib
from os.path import basename
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import COMMASPACE, formatdate, formataddr, parseaddr
from email.header import Header
from sachima import conf

mail_host = conf.get("MAIL_HOST")
MAIL_ADD = conf.get("MAIL_ADD")
MAIL_USER = conf.get("MAIL_USER")
MAIL_PASS = conf.get("MAIL_PASS")
MAIL_SENDER = conf.get("MAIL_SENDER")


def _format_addr(s):
    name, addr = parseaddr(s)
    return formataddr((Header(name, 'utf-8').encode(), addr))


def send_mail(send_to, cc_to, subject, text, files=None,
              server=mail_host, ishtml=False):
    assert isinstance(send_to, list)

    msg = MIMEMultipart()
    msg['From'] = _format_addr(MAIL_SENDER+' <%s>' % MAIL_ADD)
    msg['To'] = COMMASPACE.join([_format_addr(x) for x in send_to])
    msg['CC'] = COMMASPACE.join([_format_addr(x) for x in cc_to])
    # msg['Date'] = formatdate(localtime=True)
    msg['Subject'] = subject

    if ishtml:
        msg.attach(MIMEText(text, 'html', 'utf-8'))
    else:
        msg.attach(MIMEText(text))

    for f in files or []:
        with open(f, "rb") as fil:
            part = MIMEApplication(
                fil.read(),
                # Name=('gbk', '', basename(f))
                Name=basename(f)
            )
        # After the file is closed
        part['Content-Disposition'] = 'attachment; filename="%s"' % basename(f)
        msg.attach(part)

    smtp = smtplib.SMTP(server)
    smtp.login(MAIL_USER, MAIL_PASS)
    send_list = send_to + cc_to
    smtp.sendmail(MAIL_ADD, send_list, msg.as_string())
    smtp.close()



import redis
from sachima import conf


class Singleton(type):
    """
    An metaclass for singleton purpose. Every singleton class should inherit from this class by 'metaclass=Singleton'.
    """

    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(
                *args, **kwargs
            )
        return cls._instances[cls]


class RedisClient(metaclass=Singleton):
    def __init__(self):
        self.pool = redis.ConnectionPool(
            host=conf.get("REDIS_HOST"),
            port=conf.get("REDIS_PORT"),
            decode_responses=True
            # password=conf.get("REDIS_PASS"),
        )

    @property
    def conn(self):
        if not hasattr(self, "_conn"):
            self.getConnection()
        return self._conn

    def getConnection(self):
        self._conn = redis.StrictRedis(
            connection_pool=self.pool, charset="utf-8"
        )
